;****************************************************************
;
; TMS Demo - MSX ver 1.00 (C) Electric Adventures 2019
;
;****************************************************************
FNAME "TMSDemo.ROM"
cpu z80
ORG 4000h

INCLUDE "MSXRom-Include.asm"

PATSIZE:   EQU 73
SPRITECOUNT: EQU 1

db "AB"
DW INIT
DW 0
DW 0
DW 0
DS 6

; This is the start of your application
INIT:
    ; set screen mode immediately, as interrupts are enabled on entry
    call SETSCREEN2

    CALL INITRAM

    ; initialise clock
    LD  HL,TIMER_TABLE
    LD  DE,TIMER_DATA_BLOCK
    CALL INIT_TIMER

    ;Enable timers - Tick, 1/4 sec and 1/2 sec timers
    CALL CREATE_TIMERS

    CALL SETINTERRUPT

    ; Seed random numbers with a fixed number (nothing else to use?)
    LD HL,1967
    CALL SEED_RANDOM


    ; Do all our VRAM setup
    ; NMI is currently disabled

TITLESCREEN:
    ; display our title screen
    DI
    ; Clear the screen
    CALL CLEARPAT

    ; Clean up in case the game left anything on screen
    CALL CLEARSPRITES
    CALL SPRWRT

    ; Load the character set, make all three sections the same
    CALL LOAD_CHR_SET

    ; now setup our initial screen layout
    ; top 1/3 of the screen
    LD HL,VRAM_NAME
    LD BC,256
    LD A,69
    CALL FILVRM

    ; middle 1/3 of the screen
    LD HL,VRAM_NAME + 256
    LD BC,256
    LD A,70
    CALL FILVRM

    ; last 1/3 of the screen
    LD HL,VRAM_NAME + 512
    LD BC,256
    LD A,71
    CALL FILVRM

    ; our initial text
    LD HL,VRAM_NAME + 32*12 + 10
    CALL SEWRT
    LD HL,TITLE_TEXT
    CALL OUTPUT_TEXT

    ; add a box of characters around the text
    LD HL,VRAM_NAME + 256 + 32*2 + 6
    LD BC,20
    LD A,72
    CALL FILVRM
    LD HL,VRAM_NAME + 256 + 32*3 + 6
    CALL SETWRT
    LD A,72
    OUT (DATA_PORT),A
    LD HL,VRAM_NAME + 256 + 32*3 + 25
    CALL SETWRT
    LD A,72
    OUT (DATA_PORT),A
    LD HL,VRAM_NAME + 256 + 32*4 + 6
    CALL SETWRT
    LD A,72
    OUT (DATA_PORT),A
    LD HL,VRAM_NAME + 256 + 32*4 + 25
    CALL SETWRT
    LD A,72
    OUT (DATA_PORT),A
    LD HL,VRAM_NAME + 256 + 32*5 + 6
    CALL SETWRT
    LD A,72
    OUT (DATA_PORT),A
    LD HL,VRAM_NAME + 256 + 32*5 + 25
    CALL SETWRT
    LD A,72
    OUT (DATA_PORT),A
    LD HL,VRAM_NAME + 256 + 32*6 + 6
    LD BC,20
    LD A,72
    CALL FILVRM

   	LD HL,OUTPUT_VDP_TITLE
	CALL SET_VDU_HOOK
    EI

SPLASH_TITLE2:
    LD	A,(HalfSecTimer)
	CALL	TEST_SIGNAL
	OR	A
    JR Z,SPLASH_TITLE2
    ; Any other actions on the title screen go here
	JR SPLASH_TITLE2

; Load out character set from ROM, make all three sections the same
LOAD_CHR_SET:
    LD HL,0
SLOOP:
    LD DE,TILESET_1_PAT
    PUSH HL
    LD BC,PATSIZE*8
    CALL LDIRVM
    POP HL
    ; now load colour attributes
    PUSH HL
    LD BC,VRAM_COLOR
    ADD HL,BC
    LD DE,TILESET_1_COL
    LD BC,PATSIZE*8
    CALL LDIRVM
    POP HL
    LD BC,800h
    ADD HL,BC
    LD A,H
    CP 18h
    JR C,SLOOP
    RET

; Initialise any RAM we will be using
INITRAM:
    LD A,204
    LD (LASTPATTERN1),A
    XOR A
    LD (LASTPATTERN2),A
    LD A,045h
    LD (LASTPATTERN3),A
    LD A,8
    LD (WAIT),A
    RET

; This is our routine called every VDP interrupt during our main loop
; - Do all VDP writes here to avoid corruption
VDU_WRITES:
    RET

; This is our routine called every VDP interrupt during the title screen
; - Do all VDP writes here to avoid corruption
OUTPUT_VDP_TITLE:
    ; do our pattern and colour animations
    LD A,(WAIT)
    CP 0
    JR Z, OVT1
    DEC A
    LD (WAIT),A
    RET
OVT1:
    LD A,8
    LD (WAIT),A
    ; 1. animate the pattern in tile 69
    LD HL,69*8
    CALL SETWRT
    LD B,4
    LD A,(LASTPATTERN1)
LP1:
    OUT (DATA_PORT),A
    OUT (DATA_PORT),A
    XOR 0ffh
    DJNZ LP1
    XOR 0ffh
    LD (LASTPATTERN1),A

    ; 2. animate the palette entries on the 2nd zone
    LD HL,VRAM_COLOR + 0800h + 70 * 8
    CALL SETWRT
    LD A,(LASTPATTERN2)
    INC A
    CP 3
    JR NZ,OVT2
    XOR A
OVT2:
    LD (LASTPATTERN2),A
    LD HL,COLOURTABLE
    LD C,A
    LD B,0
    XOR A
    ADC HL,BC
    LD A,(HL)
    OUT (DATA_PORT),A
    OUT (DATA_PORT),A
    OUT (DATA_PORT),A
    INC HL
    LD A,(HL)
    OUT (DATA_PORT),A
    OUT (DATA_PORT),A
    OUT (DATA_PORT),A
    INC HL
    LD A,(HL)
    OUT (DATA_PORT),A
    OUT (DATA_PORT),A

    ; 3. animate the palette entries in the 3rd zone
    LD A,(LASTPATTERN3)
    ; swap upper and lower number by rotating 4 times
    RLCA
    RLCA
    RLCA
    RLCA
    LD (LASTPATTERN3),A
    LD HL,VRAM_COLOR + 01000h + 71*8
    LD BC,8
    CALL FILVRM
    RET

COLOURTABLE:
    DB 096,128,144,096,128

; output characters to the name table
; Video Ram write position already should be set
; HL = Pointer to text buffer to write, end at FFh
OUTPUT_TEXT:
    LD A,(HL)
    CP 255
    RET Z
    OUT (DATA_PORT),A
    INC HL
    JR OUTPUT_TEXT

; Title text
TITLE_TEXT:
db 30,23,29,0,14,41,49,51,63,255

;**************************************************************************************************
; Include external data files
;**************************************************************************************************

include "MSXRom-TMSDemo-Patterns.ASM"


;**************************************************************************************************
; Standard Libraries
;**************************************************************************************************

include "MSXRom-Lib.ASM"

END:    EQU $

;**************************************************************************************************
; RAM Definitions
;**************************************************************************************************

ORG RAMSTART

WAIT: ds 1
LASTPATTERN1: ds 1
LASTPATTERN2: ds 1
LASTPATTERN3: ds 1

