;****************************************************************
;
; Spectravideo SUBROUTINES ver 1.0 (C) Electric Adventures 2015
;
;****************************************************************

; Set address called inside NMI routine
; HL = Hook Address
SET_VDU_HOOK:
   LD A,0cdh
   LD (VDU_HOOK),A
   LD (VDU_HOOK+1),HL
   LD A,0c9h
   LD (VDU_HOOK+3),A
   RET

; Disable the generation of NMI calls
DISABLE_NMI:
    ld      a,(0f3e0h)
    and     0dfh
DNMI1:
    ld      c,a
    ld      b,1
    jp      WRITE_REGISTER

; Enable the generation of NMI calls
ENABLE_NMI:
    ld      a,(0f3e0h)
    or      020h
    call    DNMI1
    jp      WRITE_REGISTER

; Write to VDP, port in B, value in C
WRTVDP:
    LD A,B
    OUT (CTRL_PORT),A
    LD A,C
    OR 80h
    OUT (CTRL_PORT),A
    ; copy current register value to RAM
    PUSH HL
    LD A,B
    LD B,0
    LD HL,0F3DFh
    ADD HL,BC
    LD (HL),A
    POP HL
    RET

; Set write to Video Ram
; HL = VRAM Address
SETWRT:
    LD A,L
    OUT (CTRL_PORT),A
    LD A,H
    AND 3Fh
    OR 40h
    OUT (CTRL_PORT),A
    RET
;
; Set read to Video Ram
; HL = VRAM Address
SETRD:
    LD A,L
    OUT (CTRLR_PORT),A
    LD A,H
    AND 3Fh
    OUT (CTRLR_PORT),A
    RET

; Load a block of memory to VRAM
; HL = VRAM Address
; DE = RAM Address
; BC = Length
LDIRVM:
    CALL SETWRT
LLOOP:
    LD A,(DE)
    OUT (DATA_PORT),A
    INC DE
    DEC BC
    LD A,C
    OR B
    CP 0
    JR NZ,LLOOP
    RET

; Fill a section of VRAM with value in A
; HL = VRAM Address
; BC = Length
FILVRM:
    LD E,A
    CALL SETWRT
FLOOP:
    LD A,E
    OUT (DATA_PORT),A
    DEC BC
    LD A,C
    OR B
    CP 0
    JR NZ,FLOOP
    RET

;
; Write Sprite positions to VRAM
; Writes sprites in normal order for 1st call and then
; reverse order for 2nd call.  Allows 8 sprites per line
; with minimal flickering.
SPRWRT:
    LD A,(SPRORDER)
    BIT 0,A
    JR NZ,SW1
    ; write sprites normal order
    SET 0,A
    LD (SPRORDER),A
    LD HL,VRAM_SPRATTR
    LD DE,SPRTBL
    LD BC,80h
    CALL LDIRVM
    RET
SW1:
    ; write sprites reverse order
    RES 0,A
    LD (SPRORDER),A
    LD HL,VRAM_SPRATTR
    CALL SETWRT
    LD IX,SPRTBL+80h-4
    LD B,32
SW2:
    LD A,(IX+0)
    OUT (DATA_PORT),A
    LD A,(IX+1)
    OUT (DATA_PORT),A
    LD A,(IX+2)
    OUT (DATA_PORT),A
    LD A,(IX+3)
    OUT (DATA_PORT),A
    DEC IX
    DEC IX
    DEC IX
    DEC IX
    DJNZ SW2
    RET

READ_REGISTER:
    IN A,(CTRLR_PORT)
    RET

WRITE_REGISTER:
    DI
    LD A,C
    OUT (CTRL_PORT),A
    LD A,B
    AND 07h
    OR 080h
    OUT (CTRL_PORT),A
    EI
    ; copy current register value to RAM
    PUSH HL
    LD A,C
    LD C,B
    LD B,0
    LD HL,0F3DFh
    ADD HL,BC
    LD (HL),A
    POP HL
    RET

; Setup Screen 2,2 - Interrupts are enabled
SETSCREEN2:
    LD BC,0002h;Reg 0: Mode 2
    CALL WRITE_REGISTER
    LD BC,020eh        ; Name table 1800h 3800h
    CALL WRITE_REGISTER
    LD BC,03ffh        ; Colour table 2000h
    CALL WRITE_REGISTER
    LD BC,0403h        ; Pattern table 0000h - only one set of 256 characters
    CALL WRITE_REGISTER
    LD BC,0576h        ; Sprite attribute table 3b00h
    CALL WRITE_REGISTER
    LD BC,0603h        ; Sprite pattern table 1800h
    CALL WRITE_REGISTER
    LD BC,0700h        ; Base colours
    CALL WRITE_REGISTER
    LD A,0e2h
    LD (0f3e0h),A
    LD BC,01e2h ;Reg 1: Mode 2, 16k, no interrupts, 16x16 sprites c2
    CALL WRITE_REGISTER
    RET

; Output a character to the screen nametable
; (HL) contains the character to output
;
PRINTIT:
    XOR A ; clear A
    RLD   ; rotate left out of (HL) into A
    INC A
    OUT (DATA_PORT),A
    DEC A
    RLD   ; rotate left out of (HL) into A
    INC A
    OUT (DATA_PORT),A
    DEC A
    RLD
    RET

; Clear the sprites from the screen (set Y=209)
CLEARSPRITES:
    LD B,80h
    LD DE,SPRTBL
CS1:
    LD A,209
    LD (DE),A
    INC DE
    DEC B
    LD A,B
    CP 0
    JR NZ,CS1

; Clear the VDP Pattern table (clears screen)
CLEARPAT:
    LD HL,VRAM_NAME
    LD BC,768
    XOR A
    CALL FILVRM
    RET

; Check joystick 1 button
; Bit 0 of A is set if Joystick 1, button 1 is pressed
JOYTST1:
    IN A,(PPI_PORTA)
    BIT 4,A
    LD A,0
    RET NZ
    LD A,1
    RET

; Check joystick 2 button
; Bit 0 of A is set if Joystick 2, button 1 is pressed
JOYTST2:
    IN A,(PPI_PORTA)
    BIT 5,A
    LD A,0
    RET NZ
    LD A,1
    RET


; Get joystick direction
; Result in A
; Each bit is cleared if signal is true
; Bit 0 - Joystick 1 is up
; Bit 1 - Joystick 1 is down
; Bit 2 - Joystick 1 is left
; Bit 3 - Joystick 1 is right
; Bit 4 - Joystick 2 is up
; Bit 5 - Joystick 2 is down
; Bit 6 - Joystick 2 is left
; Bit 7 - Joystick 2 is right
JOYDIR:
    LD A,0Eh
    OUT (PSG_LATCH),A
    IN A,(PSG_READ)
    RET

; Play a sound from a double byte stream of data
; byte 1 = register
; byte 2 = value
SOUND:
    LD A,(HL)
    CP 255
    RET Z
    OUT (PSG_LATCH),A
    INC HL
    CP 7
    LD A,(HL)
    JR NZ,SNDLP1
    ; preserve bits 6 & 7 of register 7
    LD B,A
    IN A,(PSG_READ)
    AND 192
    ADD A,B
SNDLP1:
    OUT (PSG_WRITE),A
    INC HL
    JR SOUND

;   Seed Random numbers
;   Seed in HL
SEED_RANDOM:
    LD (SEED),HL
    RR H
    RL L
    LD (SEED+2),HL
    RET
;
;   Generate a random number, based on the initial Seed
;   value.
;
RND:
    PUSH HL
    PUSH BC
    PUSH DE
    LD DE,(SEED+2)
    LD HL,(SEED)
    LD B,5
RLP1:
    RR H
    RL L
    RR D
    RL E
    DJNZ RLP1
    LD B,3
RLP2:
    PUSH DE
    LD DE,(SEED)
    OR A
    SBC HL,DE
    EX DE,HL
    POP HL
    DJNZ RLP2
    LD (SEED),HL
    LD (SEED+2),DE
    LD A,E
    ;OR H
    POP DE
    POP BC
    POP HL
    RET
    
; NMI Interrupt Routine
; Non maskable interrupt used for:
;   music, processing timers, sprite motion processing
NMI:
    EX	AF,AF'
    EXX
    CALL	READ_REGISTER	;Side effect allows another NMI to happen
    ; update our time counter
    LD HL,(TIME)
    DEC HL
    LD (TIME),HL
    ; write sprite table
    CALL    SPRWRT
    LD A,(VDU_HOOK)
    CP 0cdh
    JR NZ,NMI2
    CALL VDU_HOOK
NMI2:
    CALL	TIME_MGR

;Now restore everything
NMI3:
    EXX
    EX	AF,AF'
    EI
    RETN

; Create and enable standard timers
CREATE_TIMERS:
    LD	HL,(AMERICA)	;How long a second is
    SRA L
    LD	H,0
    LD	A,1	;set to repeating
    CALL	REQUEST_SIGNAL
    LD	(HalfSecTimer),A		;Happens once per half second
    LD	HL,(AMERICA)	;How long a second is
    SRA L
    SRA L
    LD	H,0
    LD	A,1	;set to repeating
    CALL	REQUEST_SIGNAL
    LD	(QtrSecTimer),A		;Happens once per quarter second
    LD	HL,1
    LD	A,1	;set to repeating
    CALL	REQUEST_SIGNAL
    LD	(TickTimer),A		;Happens once per tick
    RET

;
; Timer Routines
;

DONE:    EQU 7
REPEAT:  EQU 6
FREE:    EQU 5
EOT:     EQU 4
LONG:    EQU 3
AMERICA: DB 50

TIME_MGR:
    ld HL,(TIMER_TABLE_BASE)
NEXT_TIMER0:
    bit FREE,(HL)
    call z,DCR_TIMER
    bit EOT,(HL)
    jr nz,SCRAM
    inc HL
    inc HL
    inc HL
    jr NEXT_TIMER0
SCRAM:
    ret
;
DCR_TIMER:
    push HL
    bit LONG,(HL)
    jr z,DCR_S_MODE_TBL
    bit REPEAT,(HL)
    jr nz,DCR_L_RPT_TBL
DCR_L_MODE_TBL:
    inc HL
    ld E,(HL)
    inc HL
    ld D,(HL)
    dec DE
    ld A,E
    or D
    jr nz,SAVE_2_BYTES
    pop HL
    push HL
    jr SET_DONE_BIT
;
DCR_L_RPT_TBL:
    inc HL
    ld E,(HL)
    inc HL
    ld D,(HL)
    ex de,hl
    ld e,(hl)
    inc hl
    ld d,(hl)
    dec de
    ld a,e
    or d
    jr nz,SAVE_2_BYTES
    inc hl
    ld e,(hl)
    inc hl
    ld d,(hl)
    dec hl
    dec hl
    ld (hl),d
    dec hl
    ld (hl),e
    pop hl
    push hl
    jr SET_DONE_BIT
;
DCR_S_MODE_TBL:
    inc hl
    dec (hl)
    jr nz,TIMER_EXIT
    pop hl
    push hl
    bit REPEAT,(hl)
    jr z,SET_DONE_BIT
    inc hl
    inc hl
    ld a,(hl)
    dec hl
    ld (hl),a
    dec hl
    pop hl
    push hl
SET_DONE_BIT:
    set DONE,(hl)
TIMER_EXIT:
    pop hl
    ret
;
SAVE_2_BYTES:
    ld (hl),d
    dec hl
    ld (hl),e
    jr TIMER_EXIT
;
; Procedure Init Timer
; HL has address of Timer table
; DE has address of Timer Data table
;
INIT_TIMER_PARAM:
    dw 00002h
    dw 00002h
    dw 00002h
;
INIT_TIMER:
    ld (TIMER_TABLE_BASE),hl
    ld (hl),030h
    ex de,hl
    ld (NEXT_TIMER_DATA_BYTE),hl
    ret
;
FREE_SIGNAL:
    ld c,a
    ld hl,(TIMER_TABLE_BASE)
    ld b,a
    ld de,00003h
    or a
    jr z,FREE_MATCH
FREE1:
    bit EOT,(hl)
    jr nz,FREE_EXIT
    add hl,de
    dec c
    jr nz,FREE1
FREE_MATCH:
    bit FREE,(hl)
    jr nz,FREE_SET
    set FREE,(hl)
    bit REPEAT,(hl)
    jr z,FREE_SET
    bit LONG,(hl)
    jr z,FREE_SET
; FREE (DELETE) COUNTER
; Counter address in DE
FREE_COUNTER:
    inc hl
    ld e,(hl)
    inc hl
    ld d,(hl)
    push de
    ld hl,(TIMER_TABLE_BASE)
    push hl
NEXT:
    bit EOT,(hl)
    jr nz,MOVE_IT
    bit FREE,(hl)
    jr nz,GET_NEXT
    ld a,(hl)
    and 048h
    cp 048h
    jr nz,GET_NEXT
    inc hl
    inc hl
    ld a,(hl)
    cp d
    jr c,GET_NEXT
    jr nz,SUBSTRACT_4
    dec hl
    ld a,(hl)
    cp e
    jr c,GET_NEXT
    jr z,FREE_EXIT
    inc hl
SUBSTRACT_4:
    ld d,(hl)
    dec hl
    ld e,(hl)
    dec de
    dec de
    dec de
    dec de
    ld (hl),e
    inc hl
    ld (hl),d
    ;jr GET_NEXT
;
GET_NEXT:
    pop hl
    inc hl
    inc hl
    inc hl
    push hl
    jr NEXT
;
MOVE_IT:
    ld b,000h
    or a
    pop hl
    pop de
    push hl
    ld hl,(NEXT_TIMER_DATA_BYTE)
    sbc hl,de
    ld c,l
    ld l,e
    ld h,d
    inc hl
    inc hl
    inc hl
    inc hl
    ldir
    ld bc,00008h
    sbc hl,bc
    ld (NEXT_TIMER_DATA_BYTE),hl
    pop hl
FREE_SET:
FREE_EXIT:
    ret
;
REQUEST_SIGNAL:
    ld c,a
    ex de,hl
    ld hl,(TIMER_TABLE_BASE)
    xor a
    ld b,a
TIMER1:
    bit FREE,(hl)
    jr z,NEXT_TIMER1
    push hl
    ld a,(hl)
    and 010h
    or 020h
    ld (hl),a
    xor a
    or d
    jr nz,LONG_TIMER
    or c
    jr z,NOT_A_REPEAT_TIMER
    set REPEAT,(hl)
NOT_A_REPEAT_TIMER:
    inc hl
    ld (hl),e
    inc hl
    ld (hl),e
    jr INIT_TIMER_EXIT
;
LONG_TIMER:
    set LONG,(hl)
    ld a,c
    or a
    jr z,NOT_A_LONG_REPEAT
    push de
    ex de,hl
    ld hl,(NEXT_TIMER_DATA_BYTE)
    ex de,hl
    set REPEAT,(hl)
    inc hl
    ld (hl),e
    inc hl
    ld (hl),d
    ex de,hl
    pop de
    ld (hl),e
    inc hl
    ld (hl),d
    inc hl
    ld (hl),e
    inc hl
    ld (hl),d
    inc hl
    ld (NEXT_TIMER_DATA_BYTE),hl
    jr INIT_TIMER_EXIT
;
NOT_A_LONG_REPEAT:
    inc hl
TIMER2:
    ld (hl),e
    inc hl
    ld (hl),d
    inc hl
    jr INIT_TIMER_EXIT
;
NEXT_TIMER1:
    bit EOT,(hl)
    jr nz,MAKE_NEW_TIMER
    inc hl
    inc hl
    inc hl
    inc b
    JR TIMER1
;
MAKE_NEW_TIMER:
    push de
    push hl
    inc hl
    inc hl
    inc hl
    inc b
    ld (hl),030h
    ex de,hl
    pop hl
    res EOT,(hl)
    ex de,hl
    pop de
    jr TIMER1
;
INIT_TIMER_EXIT:
    pop hl
    res FREE,(hl)
    ld a,b
    ret
;
; Procedure Test Signal
; A has the Signal number to be tested
; A value of True(1) or False(0) is returned in A
;
TEST_SIGNAL:
    ld c,a
    ld hl,(TIMER_TABLE_BASE)
    ld b,a
    ld de,00003h
    or a
    jr z,SIGNAL_MATCH
TEST1:
    bit EOT,(hl)
    jr nz,SIGNAL_FALSE
    add hl,de
    dec c
    jr nz,TEST1
SIGNAL_MATCH:
    bit FREE,(hl)
    jr nz,SIGNAL_FALSE
    bit DONE,(hl)
    jr nz,SIGNAL_TRUE
SIGNAL_FALSE:
    xor a
    jr TEST_EXIT
;
SIGNAL_TRUE:
    bit REPEAT,(hl)
    jr nz,SIGNAL_TRUE1
    set FREE,(hl)
SIGNAL_TRUE1:
    res DONE,(hl)
    ld a,001h
TEST_EXIT:
    or a
    ret

; Set origin in High RAM area,
;  - ensures works with SV-318 i.e. 16k
ORG 0C000h

TickTimer:    DS 1 ; Signal that 3 frames has elapsed
HalfSecTimer: DS 1 ; Signal that 1/2 second has elapsed
QtrSecTimer:  DS 1 ; Signal that 1/4 second has elapsed
TIME:         DS 2
SEED:	      DS 4

; Sprite positions
SPRTBL:       DS 80h
SPRORDER:     DS 1 ; Sprite write order flag

TIMER_TABLE:	    DS 16	;Pointer to timers table (16 timers)
TIMER_DATA_BLOCK:	DS 58	;Pointer to timers table for long timers

VDU_HOOK:     DS 4 ; NMI VDU Delayed writes hook

TIMER_LENGTH: DS 1
TEST_SIG_NUM: DS 1
TIMER_TABLE_BASE: DS 2
NEXT_TIMER_DATA_BYTE: DS 2

RAMSTART: EQU $ ; Setup where game specific values can start
